package jalview.ws;

import jalview.datamodel.Alignment;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.DBRefSource;
import jalview.datamodel.SequenceI;
import jalview.ws.seqfetcher.ASequenceFetcher;
import jalview.ws.seqfetcher.DbSourceProxy;

import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

public class SequenceFetcherTest
{

  /**
   * simple run method to test dbsources.
   * 
   * @param argv
   */
  public static void main(String[] argv)
  {
    // TODO: extracted from SequenceFetcher - convert to proper unit test with
    // assertions

    AlignmentI ds = null;
    Vector<Object[]> noProds = new Vector<Object[]>();
    String usage = "SequenceFetcher.main [-nodas] [<DBNAME> [<ACCNO>]]\n"
            + "With no arguments, all DbSources will be queried with their test Accession number.\n"
            + "With one argument, the argument will be resolved to one or more db sources and each will be queried with their test accession only.\n"
            + "If given two arguments, SequenceFetcher will try to find the DbFetcher corresponding to <DBNAME> and retrieve <ACCNO> from it.\n"
            + "The -nodas option will exclude DAS sources from the database fetchers Jalview will try to use.";
    boolean withDas = true;
    if (argv != null && argv.length > 0
            && argv[0].toLowerCase().startsWith("-nodas"))
    {
      withDas = false;
      String targs[] = new String[argv.length - 1];
      System.arraycopy(argv, 1, targs, 0, targs.length);
      argv = targs;
    }
    if (argv != null && argv.length > 0)
    {
      List<DbSourceProxy> sps = new SequenceFetcher(withDas)
              .getSourceProxy(argv[0]);
  
      if (sps != null)
      {
        for (DbSourceProxy sp : sps)
        {
          AlignmentI al = null;
          try
          {
            al = sp.getSequenceRecords(argv.length > 1 ? argv[1] : sp
                    .getTestQuery());
          } catch (Exception e)
          {
            e.printStackTrace();
            System.err.println("Error when retrieving "
                    + (argv.length > 1 ? argv[1] : sp.getTestQuery())
                    + " from " + argv[0] + "\nUsage: " + usage);
          }
          SequenceI[] prod = al.getSequencesArray();
          if (al != null)
          {
            for (int p = 0; p < prod.length; p++)
            {
              System.out.println("Prod " + p + ": "
                      + prod[p].getDisplayId(true) + " : "
                      + prod[p].getDescription());
            }
          }
        }
        return;
      }
      else
      {
        System.err.println("Can't resolve " + argv[0]
                + " as a database name. Allowed values are :\n"
                + new SequenceFetcher().getSupportedDb());
      }
      System.out.println(usage);
      return;
    }
    ASequenceFetcher sfetcher = new SequenceFetcher(withDas);
    String[] dbSources = sfetcher.getSupportedDb();
    for (int dbsource = 0; dbsource < dbSources.length; dbsource++)
    {
      String db = dbSources[dbsource];
      // skip me
      if (db.equals(DBRefSource.PDB))
      {
        continue;
      }
      for (DbSourceProxy sp : sfetcher.getSourceProxy(db))
      {
        System.out.println("Source: " + sp.getDbName() + " (" + db
                + "): retrieving test:" + sp.getTestQuery());
        AlignmentI al = null;
        try
        {
          al = sp.getSequenceRecords(sp.getTestQuery());
          if (al != null && al.getHeight() > 0)
          {
            boolean dna = sp.isDnaCoding();
            // try and find products
            String types[] = jalview.analysis.CrossRef
                    .findSequenceXrefTypes(dna, al.getSequencesArray());
            if (types != null)
            {
              System.out.println("Xref Types for: "
                      + (dna ? "dna" : "prot"));
              for (int t = 0; t < types.length; t++)
              {
                System.out.println("Type: " + types[t]);
                SequenceI[] prod = jalview.analysis.CrossRef
                        .findXrefSequences(al.getSequencesArray(), dna,
                                types[t], null)
                        .getSequencesArray();
                System.out.println("Found "
                        + ((prod == null) ? "no" : "" + prod.length)
                        + " products");
                if (prod != null)
                {
                  for (int p = 0; p < prod.length; p++)
                  {
                    System.out.println("Prod " + p + ": "
                            + prod[p].getDisplayId(true));
                  }
                }
              }
            }
            else
            {
              noProds.addElement((dna ? new Object[] { al, al }
                      : new Object[] { al }));
            }
  
          }
        } catch (Exception ex)
        {
          System.out.println("ERROR:Failed to retrieve test query.");
          ex.printStackTrace(System.out);
        }
  
        if (al == null)
        {
          System.out.println("ERROR:No alignment retrieved.");
          StringBuffer raw = sp.getRawRecords();
          if (raw != null)
          {
            System.out.println(raw.toString());
          }
          else
          {
            System.out.println("ERROR:No Raw results.");
          }
        }
        else
        {
          System.out.println("Retrieved " + al.getHeight() + " sequences.");
          for (int s = 0; s < al.getHeight(); s++)
          {
            SequenceI sq = al.getSequenceAt(s);
            while (sq.getDatasetSequence() != null)
            {
              sq = sq.getDatasetSequence();
  
            }
            if (ds == null)
            {
              ds = new Alignment(new SequenceI[] { sq });
  
            }
            else
            {
              ds.addSequence(sq);
            }
          }
        }
        System.out.flush();
        System.err.flush();
  
      }
      if (noProds.size() > 0)
      {
        Enumeration<Object[]> ts = noProds.elements();
        while (ts.hasMoreElements())
  
        {
          Object[] typeSq = ts.nextElement();
          boolean dna = (typeSq.length > 1);
          AlignmentI al = (AlignmentI) typeSq[0];
          System.out.println("Trying getProducts for "
                  + al.getSequenceAt(0).getDisplayId(true));
          System.out.println("Search DS Xref for: "
                  + (dna ? "dna" : "prot"));
          // have a bash at finding the products amongst all the retrieved
          // sequences.
          SequenceI[] seqs = al.getSequencesArray();
          Alignment prodal = jalview.analysis.CrossRef.findXrefSequences(
                  seqs, dna, null, ds);
          System.out.println("Found "
                  + ((prodal == null) ? "no" : "" + prodal.getHeight())
                  + " products");
          if (prodal != null)
          {
            SequenceI[] prod = prodal.getSequencesArray(); // note
            // should
            // test
            // rather
            // than
            // throw
            // away
            // codon
            // mapping
            // (if
            // present)
            for (int p = 0; p < prod.length; p++)
            {
              System.out.println("Prod " + p + ": "
                      + prod[p].getDisplayId(true));
            }
          }
        }
  
      }
  
    }
  }

}
